@@ -21,10 +21,14 @@ UCI flying 200 time trial.
 
 Peculiarities when compared to ittt:
 
- - times are truncated to thousandths
+ - times are _truncated_ to thousandths
  - dead heat on thousandths is broken by time over last 100m
  - autoarm is the default
 
+TODO: 
+ - use sectormap to track and report rider speed
+ - report split/rank over each sector
+
 """
 
 import gtk
@@ -65,8 +69,8 @@ COL_100 = 9		# 100m split ToD
 # scb consts
 SCB_NAME_WIDTH = metarace.SCB_LINELEN - 9
 SCB_RESNAME_WIDTH = metarace.SCB_LINELEN - 11
-SCB_RESULT_FMT = [(2, 'l'), (3, 'r'), ' ',
-                  (SCB_RESNAME_WIDTH, 'l'), ' ', (4, 'r')]
+SCB_RESULT_FMT = [(2, u'l'), (3, u'r'), u' ',
+                  (SCB_RESNAME_WIDTH, u'l'), u' ', (4, u'r')]
 
 # scb function key mappings (also trig announce)
 key_reannounce = 'F4'                # (+CTRL) calls into delayed announce
@@ -86,7 +90,7 @@ key_abort = 'F7'		     # + ctrl abort A
 
 class f200ranks(object):
     """Helper class for managing flying 200 ranks."""
-    def __init__(self, lbl=''):
+    def __init__(self, lbl=u''):
         self.__label = lbl
         self.__store = []
 
@@ -99,7 +103,7 @@ class f200ranks(object):
     def __getitem__(self, key):
         return self.__store[key]
 
-    def rank(self, bib, series=''):
+    def rank(self, bib, series=u''):
         """Return current 0-based rank for given bib."""
         ret = None
         count = 0
@@ -122,7 +126,7 @@ class f200ranks(object):
     def clear(self):
         self.__store = []
 
-    def remove(self, bib, series=''):
+    def remove(self, bib, series=u''):
         i = 0
         while i < len(self.__store):
             if (self.__store[i][0].refid == bib
@@ -131,7 +135,7 @@ class f200ranks(object):
             else:
                 i += 1
 
-    def insert(self, t, s=None, bib=None, series=''):
+    def insert(self, t, s=None, bib=None, series=u''):
         """Insert t into ordered list."""
         ret = None
         trunc = True
@@ -217,17 +221,30 @@ class f200(object):
         self.meet.scbwin = None
         self.timerwin = False     # TODO: bib width enhancement
         fmtplaces = []
+        self.meet.announce.gfx_overlay(1)
+        self.meet.announce.gfx_set_title(u'Result: '
+                            + self.event[u'pref'] + u' '
+                            + self.event[u'info'])
         for r in self.riders:
-            if r[COL_PLACE] is not None and r[COL_PLACE] != '':
-                if r[COL_PLACE].isdigit() and int(r[COL_PLACE]) > 20:
+            if r[COL_PLACE] is not None and r[COL_PLACE] != u'':
+                if r[COL_PLACE].isdigit() and int(r[COL_PLACE]) > 12:
                     break
+                first = r[COL_FIRSTNAME].decode('utf-8')
+                last = r[COL_LASTNAME].decode('utf-8')
+                club = r[COL_CLUB].decode('utf-8')
+                plstr = r[COL_PLACE] + u'.'
+                elapstr = ''
+                if r[COL_START] and r[COL_FINISH]:
+                    elapstr = (r[COL_FINISH]-r[COL_START]).rawtime(3)
                 fmtplaces.append([r[COL_PLACE], r[COL_BIB],
-                                  strops.fitname(r[COL_FIRSTNAME],
-                                  r[COL_LASTNAME], SCB_RESNAME_WIDTH),
-                                  r[COL_CLUB]])
+                            strops.fitname(first, last, SCB_RESNAME_WIDTH),
+                                  club])
+                self.meet.announce.gfx_add_row([plstr, 
+                            strops.resname(first, last, club),
+                            elapstr])
         self.meet.scbwin = scbwin.scbtable(self.meet.scb,
                        self.meet.racenamecat(self.event),
-                       SCB_RESULT_FMT, fmtplaces, delay=90)
+                       SCB_RESULT_FMT, fmtplaces, delay=80)
         self.meet.scbwin.reset()
 
     def todstr(self, col, cr, model, iter, data=None):
@@ -243,12 +260,12 @@ class f200(object):
             else:
                 cr.set_property('style', pango.STYLE_NORMAL)
             mstr = (ft - st).rawtime(3).rjust(6)
-            sstr = ''.rjust(7)
+            sstr = u''.rjust(7)
             if sp is not None:
-                sstr = '/' + (ft - sp).rawtime(3).ljust(6)
+                sstr = u'/' + (ft - sp).rawtime(3).ljust(6)
             cr.set_property('text', mstr+sstr)
         else:
-            cr.set_property('text', '')
+            cr.set_property('text', u'')
 
     def loadconfig(self):
         """Load race config from disk."""
@@ -274,7 +291,7 @@ class f200(object):
         cr.add_section('event')
         cr.add_section('riders')
         if os.path.isfile(self.configpath):
-            self.log.debug('Attempting to read config from '
+            self.log.debug(u'Attempting to read config from '
                                + repr(self.configpath))
             cr.read(self.configpath)
 
@@ -347,14 +364,14 @@ class f200(object):
         # an appropriate outcome.
         eid = cr.get('event', 'id')
         if eid and eid != EVENT_ID:
-            self.log.error('Event configuration mismatch: '
-                           + repr(eid) + ' != ' + repr(EVENT_ID))
+            self.log.error(u'Event configuration mismatch: '
+                           + repr(eid) + u' != ' + repr(EVENT_ID))
             self.readonly = True
 
     def saveconfig(self):
         """Save race to disk."""
         if self.readonly:
-            self.log.error('Attempt to save readonly ob.')
+            self.log.error(u'Attempt to save readonly ob.')
             return
         cw = ConfigParser.ConfigParser()
         cw.add_section('event')
@@ -474,64 +491,72 @@ class f200(object):
         ret = []
         cnt = 0
         sec = printing.dual_ittt_startlist()
-        sec.set_single()
+        sec.set_single()	# 200s are one-up
 
-        sec.heading = ' '.join([
-                      'Event', self.evno, ':',
-                  self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                  self.meet.edb.getvalue(self.event, eventdb.COL_INFO),
-                      '- Start List'])
+        sec.heading = u' '.join([
+                      u'Event', self.evno, u':',
+                      self.event[u'pref'], self.event[u'info'],
+                      u'- Start List'])
+        lapstring = strops.lapstring(self.event[u'laps'])
+        substr = u' '.join([lapstring, self.event[u'dist'],
+                             self.event[u'prog']]).strip()
+        if substr:
+            sec.subheading = substr
+        if self.event[u'reco']:
+            sec.recordstr = self.event[u'reco']
         sec.lines = self.get_heats()
-        return sec
+        ret.append(sec)
+        return ret
 
     def get_startlist(self):
         """Return a list of bibs in the rider model."""
         ret = []
         for r in self.riders:
             ret.append(r[COL_BIB])
-        return ' '.join(ret)
+        return ' '.join(ret)	# will become u' ' once config unisafe
 
     def delayed_announce(self):
         """Initialise the announcer's screen after a delay."""
         if self.winopen:
             self.meet.announce.clrall()
-            self.meet.ann_title(' '.join([
-                  'Event', self.evno, ':',
-                  self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                  self.meet.edb.getvalue(self.event, eventdb.COL_INFO)]))
+            self.meet.ann_title(u' '.join([
+                  u'Event', self.evno, u':',
+                  self.event[u'pref'], self.event[u'info']]))
 
-            self.meet.announce.linefill(1, '_')
-            self.meet.announce.linefill(7, '_')
+            self.meet.announce.linefill(1, u'_')
+            self.meet.announce.linefill(7, u'_')
 
-            # fill in front straight
+            # fill in front straight (only one?)
             fbib = self.fs.getrider()
-            if fbib is not None and fbib != '':
+            if fbib is not None and fbib != u'':
                 r = self.getrider(fbib)
                 if r is not None:
-                    clubstr = ''
-                    if r[COL_CLUB] != '' and len(r[COL_CLUB]) <= 3:
-                        clubstr = '(' + r[COL_CLUB] + ')'
+                    clubstr = u''
+                    if r[COL_CLUB] != u'' and len(r[COL_CLUB]) <= 3:
+                        clubstr = u'(' + r[COL_CLUB] + u')'
                     namestr = strops.fitname(r[COL_FIRSTNAME],
                                              r[COL_LASTNAME], 24, trunc=True)
-                    placestr = '   ' # 3 ch
-                    if r[COL_PLACE] != '':
-                        placestr = strops.truncpad(r[COL_PLACE] + '.', 3)
-                    bibstr = strops.truncpad(r[COL_BIB], 3, 'r')
-                    tmstr = ''
+                    placestr = u'   ' # 3 ch
+                    if r[COL_PLACE] != u'':
+                        placestr = strops.truncpad(r[COL_PLACE] + u'.', 3)
+                    bibstr = strops.truncpad(r[COL_BIB], 3, u'r')
+                    tmstr = u''
                     et = None
                     if r[COL_START] is not None and r[COL_FINISH] is not None:
                         et = (r[COL_FINISH] - r[COL_START]).truncate(3)
-                        tmstr = '200m: ' + et.rawtime(3).rjust(12)
-                    cmtstr = ''
+                        tmstr = u'200m: ' + et.rawtime(3).rjust(12)
+                    cmtstr = u''
                     if et is not None:
-                        cmtstr = strops.truncpad('Average: ' + et.speedstr(200), 38, 'r')
-                    elif r[COL_COMMENT] is not None and r[COL_COMMENT] != '':
+                        cmtstr = strops.truncpad(u'Average: '
+                                                + et.speedstr(200), 38, u'r')
+                    elif r[COL_COMMENT] is not None and r[COL_COMMENT] != u'':
                         cmtstr = strops.truncpad(
-                                 '[' + r[COL_COMMENT].strip() + ']', 38, 'r')
-                    self.meet.announce.postxt(3,0,'        Current Rider')
-                    self.meet.announce.postxt(4,0,' '.join([placestr, bibstr,
+                             u'[' + r[COL_COMMENT].strip() + u']', 38, u'r')
+                    self.meet.announce.postxt(3,0,u'        Current Rider')
+                    self.meet.announce.postxt(4,0,u' '.join([placestr, bibstr,
                                                          namestr, clubstr]))
-                    self.meet.announce.postxt(5,0,strops.truncpad(tmstr, 38, 'r'))
+                    self.meet.announce.postxt(5,0,strops.truncpad(tmstr,
+                                                   38, u'r'))
                     self.meet.announce.postxt(6,0,cmtstr)
 
             # fill in leaderboard/startlist
@@ -544,28 +569,28 @@ class f200(object):
                     curline = 9
                     posoft = 42
 
-                clubstr = ''
-                if r[COL_CLUB] != '' and len(r[COL_CLUB]) <= 3:
-                    clubstr = ' (' + r[COL_CLUB] + ')'
+                clubstr = u''
+                if r[COL_CLUB] != u'' and len(r[COL_CLUB]) <= 3:
+                    clubstr = u' (' + r[COL_CLUB] + u')'
  
                 namestr = strops.truncpad(strops.fitname(r[COL_FIRSTNAME],
                               r[COL_LASTNAME], 22-len(clubstr),
                                   trunc=True)+clubstr, 22, elipsis=False)
-                placestr = '   ' # 3 ch
-                if r[COL_PLACE] != '':
-                    placestr = strops.truncpad(r[COL_PLACE] + '.', 3)
-                bibstr = strops.truncpad(r[COL_BIB], 3, 'r')
-                tmstr = '       ' # 7 ch
+                placestr = u'   ' # 3 ch
+                if r[COL_PLACE] != u'':
+                    placestr = strops.truncpad(r[COL_PLACE] + u'.', 3)
+                bibstr = strops.truncpad(r[COL_BIB], 3, u'r')
+                tmstr = u'       ' # 7 ch
                 if r[COL_START] is not None and r[COL_FINISH] is not None:
                     tmstr = strops.truncpad(
-                           (r[COL_FINISH] - r[COL_START]).rawtime(3), 7, 'r')
-                self.meet.announce.postxt(curline, posoft, ' '.join([
+                           (r[COL_FINISH] - r[COL_START]).rawtime(3), 7, u'r')
+                self.meet.announce.postxt(curline, posoft, u' '.join([
                       placestr, bibstr, namestr, tmstr]))
                 curline += 1
 
     def shutdown(self, win=None, msg='Exiting'):
         """Terminate race object."""
-        self.log.debug('Race Shutdown: ' + msg)
+        self.log.debug(u'Race Shutdown: ' + msg)
         if not self.readonly:
             self.saveconfig()
         self.winopen = False
@@ -656,12 +681,17 @@ class f200(object):
 
     def result_report(self):
         """Return a list of printing sections containing the race result."""
+        self.placexfer()
         ret = []
-        sec = printing.section()
-        sec.heading = 'Event ' + self.evno + ': ' + ' '.join([
-                       self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                       self.meet.edb.getvalue(self.event, eventdb.COL_INFO)
+        sec = printing.section()	# result goes to std section
+        sec.heading = u'Event ' + self.evno + u': ' + u' '.join([
+                       self.event[u'pref'], self.event[u'info']
                               ]).strip()
+        lapstring = strops.lapstring(self.event[u'laps'])
+        substr = u' '.join([lapstring, self.event[u'dist'],
+                             self.event[u'prog']]).strip()
+        if substr:
+            sec.subheading = substr
         sec.lines = []
         ftime = None
         for r in self.riders:
@@ -670,13 +700,13 @@ class f200(object):
             rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
                                       r[COL_CLUB])
             rtime = None
-            rcat = None
+            rcat = None		# should check ev[u'cate']
             info = None
             dtime = None
             if self.onestart:
-                if r[COL_PLACE] != '':
+                if r[COL_PLACE] != u'':
                     if r[COL_PLACE].isdigit():
-                        rank = r[COL_PLACE] + '.'
+                        rank = r[COL_PLACE] + u'.'
                     else:
                         rank = r[COL_PLACE]
                 if r[COL_FINISH] is not None:
@@ -684,75 +714,29 @@ class f200(object):
                     if ftime is None:
                         ftime = time
                     else:
-                        dtime = '+' + (time - ftime).rawtime(2)
+                        dtime = u'+' + (time - ftime).rawtime(2)
                     rtime = time.rawtime(3)
 
             sec.lines.append([rank, rno, rname, None, rtime, dtime])
         ret.append(sec)
         return ret
 
-    def result_export(self, f):
-        """Export results to supplied file handle."""
-        cr = csv.writer(f)
-        df = '200m'	# fixed in this case
-        header = ['Event ' + self.evno, '', ' '.join([
-                       self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                       self.meet.edb.getvalue(self.event, eventdb.COL_INFO)
-                              ]).strip(), '',
-                  df, '100m', 'last 100']
-        cr.writerow(header)
-        for r in self.riders:
-            rno = r[COL_BIB]
-            rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
-                                      r[COL_CLUB])
-            plstr = r[COL_PLACE]
-            if plstr != '':
-                plstr = "'" + plstr
-                if r[COL_PLACE].isdigit():
-                    plstr += '.'
-            if r[COL_PLACE] != '' or r[COL_COMMENT] != '':
-                if r[COL_START] is None:
-                    r[COL_START] = tod.tod(0)
-                elap = ''
-                if r[COL_FINISH] is not None:
-                    elap = (r[COL_FINISH]-r[COL_START]).rawtime(3)
-                cmt = r[COL_COMMENT]
-                #if cmt == '':
-                    #dbr = self.meet.rdb.getrider(r[COL_BIB], self.series)
-                    #if dbr is not None:
-                        #cmt = self.meet.rdb.getvalue(dbr,
-                                                   #riderdb.COL_CAT)
-                resrow = [plstr,rno, rname, cmt, "'" + elap]
-                lt = r[COL_START]
-                if r[COL_100] is not None:
-                    resrow.append("'"
-                             + (r[COL_100]-lt).rawtime(3)) # f100
-                    if r[COL_FINISH] is not None:
-                        resrow.append("'"
-                        + (r[COL_FINISH] - r[COL_100]).rawtime(3)) # l100
-                else:
-                    resrow.append('')	# f100
-                    resrow.append('')	# l100
-                cr.writerow(resrow)
-            else:
-                cr.writerow([plstr,rno,rname,'',''])
-
     def editent_cb(self, entry, col):
         """Shared event entry update callback."""
-        if col == eventdb.COL_PREFIX:
-            self.meet.edb.editevent(self.event, prefix=entry.get_text())
-        elif col == eventdb.COL_INFO:
-            self.meet.edb.editevent(self.event, info=entry.get_text())
+        if col == u'pref':
+            self.event[u'pref'] = entry.get_text().decode('utf-8', 'replace')
+        elif col == u'info':
+            self.event[u'info'] = entry.get_text().decode('utf-8', 'replace')
         self.update_expander_lbl_cb()
 
     def update_expander_lbl_cb(self):
         """Update race info expander label."""
-        self.info_expand.set_label('Race Info : '
+        self.info_expand.set_label(u'Race Info : '
                     + self.meet.racenamecat(self.event, 64))
 
     def clear_rank(self, cb):
         """Run callback once in main loop idle handler."""
-        cb('')
+        cb(u'')
         return False
 
     def split_trig(self, sp, t):
@@ -762,12 +746,12 @@ class f200(object):
         rank = self.splits.rank(bib)
         self.log_split(sp.getrider(), self.curstart, t)
         #self.log_lap(sp.getrider(), sp.lap+1, self.curstart, t, prev)
-        sp.intermed(t, 1)
+        sp.intermed(t, 2)	# show 100m split... delay ~2 sec
         if self.timerwin and type(self.meet.scbwin) is scbwin.scbtt:
             lapstr = strops.num2ord(str(rank + 1)) + ' at 100m'
-            #self.meet.scbwin.setr1('(' + str(rank + 1) + ')')
-            #glib.timeout_add_seconds(1, self.clear_rank,
-                                        #self.meet.scbwin.setr1)
+            self.meet.scbwin.setr1('(' + str(rank + 1) + ')')
+            glib.timeout_add_seconds(2, self.clear_rank,
+                                        self.meet.scbwin.setr1)
             # announce lap and rank to uSCBsrv
             self.meet.announce.postxt(5, 8, strops.truncpad(lapstr,17)
                                       + ' ' + self.fs.ck.get_text())
@@ -803,8 +787,17 @@ class f200(object):
             self.meet.gemini.set_rank(place)
             self.meet.gemini.set_time((t - self.curstart).rawtime(2))
             self.meet.gemini.show_brt()
+            self.meet.announce.gfx_overlay(2)
+            self.meet.announce.gfx_set_title(self.event[u'pref'] + u' '
+                                + self.event[u'info'])
+            if self.fs.namevec is not None:
+                self.fs.namevec[0] = u'(' + place + u')'
+                self.fs.namevec[2] = (t - self.curstart).rawtime(3)
+                self.meet.announce.gfx_add_row(self.fs.namevec)
+
         # call for a delayed announce...
         glib.idle_add(self.delayed_announce)
+        self.meet.delayed_export()
 
     def timercb(self, e):
         """Handle a timer event."""
@@ -841,11 +834,11 @@ class f200(object):
     def show_200_ttb(self, scb):
         """Display time to beat."""
         if len(self.results) > 0:
-            scb.setr2('Fastest:')
+            scb.setr2(u'Fastest:')
             scb.sett2(self.results[0][0].timestr(3))
         return False
 
-    def clear_200_ttb(self, scb, r2='', t2=''):
+    def clear_200_ttb(self, scb, r2=u'', t2=u''):
         """Clear time to beat."""
         scb.setr2(r2)
         scb.sett2(t2)
@@ -966,7 +959,6 @@ class f200(object):
         # if reqd, do places
         if doplaces:
             self.placexfer()
-            self.meet.delayed_export()
 
     def armstart(self):
         """Arm timer for start trigger."""
@@ -1044,12 +1036,18 @@ class f200(object):
             name = '[New Rider]'
             r = self.getrider(bib)
             if r is not None and r[COL_BIB] != '':
-                name = strops.fitname(r[COL_FIRSTNAME], r[COL_LASTNAME],
+                first = r[COL_FIRSTNAME].decode('utf-8')
+                last = r[COL_LASTNAME].decode('utf-8')
+                club = r[COL_CLUB].decode('utf-8')
+                name = strops.fitname(first,
+                                      last,
                                       SCB_NAME_WIDTH)
+                tp.namevec = [bib, strops.resname(first, last, club), '']
             return ' '.join([strops.truncpad(r[COL_BIB], 3, 'r'),
                              strops.truncpad(name, SCB_NAME_WIDTH),
                              strops.truncpad(r[COL_CLUB], 4, 'r')])
         else:
+            tp.namevec = None
             return ''
         
     def showtimerwin(self):
@@ -1062,6 +1060,12 @@ class f200(object):
         self.meet.gemini.reset_fields()
         self.meet.gemini.set_bib(self.fs.getrider())
         self.meet.gemini.show_brt()
+        self.meet.announce.gfx_overlay(2)
+        self.meet.announce.gfx_set_title(self.event[u'pref'] + u' '
+                            + self.event[u'info'])
+        if self.fs.namevec is not None:
+            self.meet.announce.gfx_add_row(self.fs.namevec)
+
         self.timerwin = True
         self.meet.scbwin.reset()
 
@@ -1106,23 +1110,23 @@ class f200(object):
         r = self.getrider(bib)
         if r is None:
             if self.meet.get_clubmode():        # fill in starters
-                self.log.warn('Adding non-starter: ' + repr(bib))
+                self.log.warn(u'Adding non-starter: ' + repr(bib))
                 self.addrider(bib)
                 r = self.getrider(bib)
             else:       # 'champs' mode
                 return None
-        rtxt = '[New Rider]'
-        if r is not None and (r[COL_FIRSTNAME] != ''
-                              or r[COL_LASTNAME] != ''):
-            rtxt = r[COL_FIRSTNAME] + ' ' + r[COL_LASTNAME]
-            if r[3] != '':
-                rtxt += '(' + r[3] + ')'
+        rtxt = u'[New Rider]'
+        if r is not None and (r[COL_FIRSTNAME] != u''
+                              or r[COL_LASTNAME] != u''):
+            rtxt = r[COL_FIRSTNAME] + u' ' + r[COL_LASTNAME]
+            if r[3] != u'':
+                rtxt += u'(' + r[3] + u')'
         return rtxt
 
     def bibent_cb(self, entry, tp):
         """Bib entry callback."""
         bib = entry.get_text().strip()
-        if bib != '' and bib.isalnum():
+        if bib != u'' and bib.isalnum():
             nstr = self.lanelookup(bib)
             if nstr is not None:
                 tp.biblbl.set_text(nstr)
@@ -1131,7 +1135,7 @@ class f200(object):
                 if self.timerstat == 'running':
                     tp.start(self.curstart)
             else:
-                self.log.warn('Ignoring non-starter: ' + repr(bib))
+                self.log.warn(u'Ignoring non-starter: ' + repr(bib))
                 tp.toidle()
         else:
             tp.toidle()
@@ -1223,21 +1227,21 @@ class f200(object):
 
     def log_clear(self, bib):
         """Print clear time log."""
-        self.meet.timer_log_msg(bib, '- Time Cleared -')
+        self.meet.timer_log_msg(bib, u'- Time Cleared -')
 
     def log_split(self, bib, start, split):
         """Print 100 split log."""
-        self.meet.timer_log_straight(bib, '100', split-start, 3)
+        self.meet.timer_log_straight(bib, u'100', split-start, 3)
         
     def log_elapsed(self, bib, start, finish, split=None, manual=False):
         """Print elapsed log info."""
         if manual:
-            self.meet.timer_log_msg(bib, '- Manual Adjust -')
-        self.meet.timer_log_straight(bib, 'ST', start)
-        self.meet.timer_log_straight(bib, 'FIN', finish)
+            self.meet.timer_log_msg(bib, u'- Manual Adjust -')
+        self.meet.timer_log_straight(bib, u'ST', start)
+        self.meet.timer_log_straight(bib, u'FIN', finish)
         if split is not None:
-            self.meet.timer_log_straight(bib, 'L100', finish - split, 3)
-        self.meet.timer_log_straight(bib, 'TIME', finish - start, 3)
+            self.meet.timer_log_straight(bib, u'L100', finish - split, 3)
+        self.meet.timer_log_straight(bib, u'TIME', finish - start, 3)
 
     def destroy(self):
         """Signal race shutdown."""
@@ -1257,14 +1261,14 @@ class f200(object):
         """Constructor."""
         self.meet = meet
         self.event = event      # Note: now a treerowref
-        self.evno = meet.edb.getvalue(event, eventdb.COL_EVNO)
-        self.evtype = meet.edb.getvalue(event, eventdb.COL_TYPE)
-        self.series = meet.edb.getvalue(event, eventdb.COL_SERIES)
+        self.evno = event[u'evid']
+        self.evtype = event[u'type']
+        self.series = event[u'seri']
         self.configpath = meet.event_configfile(self.evno)
 
         self.log = logging.getLogger('f200')
         self.log.setLevel(logging.DEBUG)
-        self.log.debug('Creating new f200 event: ' + str(self.evno))
+        self.log.debug(u'Creating new f200 event: ' + self.evno)
 
         # properties
         self.distance = 200
@@ -1307,17 +1311,12 @@ class f200(object):
         b.get_object('race_info_evno').set_text(self.evno)
         self.showev = b.get_object('race_info_evno_show')
         self.prefix_ent = b.get_object('race_info_prefix')
-        self.prefix_ent.connect('changed', self.editent_cb,
-                                 eventdb.COL_PREFIX)
-        self.prefix_ent.set_text(self.meet.edb.getvalue(
-                   self.event, eventdb.COL_PREFIX))
+        self.prefix_ent.connect('changed', self.editent_cb,u'pref')
+        self.prefix_ent.set_text(self.event[u'pref'])
         self.info_ent = b.get_object('race_info_title')
-        self.info_ent.connect('changed', self.editent_cb,
-                               eventdb.COL_INFO)
-        self.info_ent.set_text(self.meet.edb.getvalue(
-                   self.event, eventdb.COL_INFO))
+        self.info_ent.connect('changed', self.editent_cb,u'info')
+        self.info_ent.set_text(self.event[u'info'])
         b.get_object('race_type').set_text('Flying 200')
-        
 
         # Timer Pane
         mf = b.get_object('race_timer_pane')
