@@ -315,6 +315,27 @@ class roadmeet:
         dlg = b.get_object('properties')
         dlg.set_transient_for(self.window)
 
+        # setup the type entry
+        tcombo = b.get_object('type_combo')
+        tmodel = b.get_object('type_model')
+        tlbl = self.etype
+        dotype = False
+        if self.etype in ROADRACE_TYPES:
+            tlbl = ROADRACE_TYPES[self.etype]
+        if self.etype in [u'rms', u'rhcp']:
+            # make the combo so switcheroo is possible
+            dotype = True
+            cnt = 0
+            for t in [u'rms', u'rhcp']:
+                tmodel.append([t, ROADRACE_TYPES[t]])
+                if t == self.etype:
+                    tcombo.set_active(cnt)
+                cnt += 1
+            tcombo.set_sensitive(True)
+        else:
+            tcombo.set_sensitive(False)
+            tmodel.append([self.etype,tlbl])
+
         # fill text entries
         t_ent = b.get_object('title_entry')
         t_ent.set_text(self.title_str)
@@ -381,6 +402,17 @@ class roadmeet:
             if nport != self.alttimer_port:
                 self.alttimer_port = nport
                 self.alttimer.setport(nport)
+
+            if dotype:
+                # check for type change
+                nt = tmodel.get_value(tcombo.get_active_iter(), 0)
+                if nt != self.etype:
+                    event = self.edb.getfirst()
+                    event[u'type'] = nt
+                    self.log.info(u'Changing event type from ' +
+                                  repr(self.etype) + u' to ' + repr(nt))
+                    self.etype = nt
+                    # is a reload required?
             self.log.debug(u'Properties updated.')
         else:
             self.log.debug(u'Edit properties cancelled.')
@@ -565,12 +597,6 @@ class roadmeet:
             if sections:
                 self.print_report(sections, title, u'')
 
-    #def menu_reports_catresult_activate_cb(self, menuitem, data=None):
-        #"""Generate the categorised race result report."""
-        #if self.curevent is not None:
-            #title = u'Result'
-            #self.print_report(self.curevent.catresult_report(), title, u'')
-
     ### TODO: if reports have options, this is the dialog, saved to meet
     def menu_reports_prefs_activate_cb(self, menuitem, data=None):
         """Run the report preferences dialog."""
