@@ -376,9 +376,10 @@ class rr_announce(object):
         return True
 
     def clear(self):
-        #self.lbl_header.set_text(self.motd)
-        #self.elap_lbl.set_text('')
-        #self.gap_lbl.set_text('')
+        self.lbl_header.set_text(self.motd)
+        self.final = False
+        self.elap_lbl.set_text('')
+        self.gap_lbl.set_text('')
         self.riders.clear()
         self.map_redraw()		# update src map
         self.map_area.queue_draw()	# queue copy to screen
@@ -390,8 +391,21 @@ class rr_announce(object):
             if rftime is not None:
                 if len(self.riders) == 0:
                     # Case 1: Starting a new lap
-                    self.cur_lap = (rftime-self.cur_split).truncate(0)
-                    self.cur_split = rftime.truncate(0)
+                    if self.lapfin is not None:
+                        self.cur_split = self.lapfin
+                    else:
+                        self.cur_split = rftime
+                    if self.lapstart is not None:
+                        self.cur_lap = (self.cur_split
+                                        - self.lapstart).truncate(0)
+                    elif self.elapstart:	# no lap start?
+                        self.cur_lap = (self.cur_split
+                                        - self.elapstart).truncate(0)
+                    else:
+                        self.cur_lap = self.cur_split.truncate(0)
+                    if self.final:
+                        # overwrite cur split for down times
+                        self.cur_split = tod.ZERO
                     self.cur_bunchid = 0
                     self.cur_bunchcnt = 1
                     self.last_time = rftime
@@ -416,7 +430,7 @@ class rr_announce(object):
                     self.cur_bunchcnt = 1
                     self.last_time = rftime
                     nr=[sr[0],sr[1],sr[2],sr[3],
-                        '+' + (rftime.truncate(0) - self.cur_split).rawtime(0),
+                        '+' + (rftime - self.cur_split).rawtime(0),
                         self.cur_bunchcnt,
                         COLOURMAP[self.cur_bunchid][0],
                         rftime]
@@ -434,14 +448,43 @@ class rr_announce(object):
         redraw = False
         if m.header == 'rider':
             self.append_rider(m.text)
-        elif m.header == 'time':
-            self.elap_lbl.set_text(m.text)
-        elif m.header == 'gap':
-            self.gap_lbl.set_text(m.text)
+        #elif m.header == 'time':
+            #self.elap_lbl.set_text(m.text)
+        #elif m.header == 'gap':
+            #self.gap_lbl.set_text(m.text)
         elif m.header == 'title':
             self.lbl_header.set_text(m.text)
         elif m.header == 'start':
-            self.cur_split = tod.str2tod(m.text)
+            self.elapstart = tod.str2tod(m.text)
+        elif m.header == 'finish':
+            self.elapfin = tod.str2tod(m.text)
+        elif m.header == 'lapstart':
+            self.lapstart = tod.str2tod(m.text)
+        elif m.header == 'lapfin':
+            self.lapfin = tod.str2tod(m.text)
+        elif m.header == u'laplbl':
+            if m.text:
+                self.laplbl = m.text
+            else:
+                self.laplbl = None      # force None for empty string
+        elif m.header == u'laptype':
+            if m.text:
+                self.laptype = m.text
+            else:
+                self.laptype = None     # force None for empty string
+        elif m.header == u'distance':
+            self.distance = None
+            try:
+                a = float(m.text)
+                if a > 0.1:
+                    self.distance = a
+            except:
+                #self.log.debug(u'Invalid distance: ' + repr(cmd.text))
+                print(u'Invalid distance: ' + repr(m.text))
+        elif m.header == u'timerstat':
+            self.timerstat = m.text
+        elif m.header == u'bunches':
+            self.final = True
         elif m.erp:
             self.clear()
         return False
@@ -534,6 +577,67 @@ class rr_announce(object):
         else:
             return False
 
+    def timeout(self):
+        """Update status."""
+        # 1: Terminate?
+        if not self.running:
+            return False
+        # 2: Process?
+        try:
+            ntime = tod.tod(u'now')
+            ntod = ntime.truncate(0)
+            if ntime >= self.nc.truncate(1):
+                self.tod = ntod
+                self.process_timeout()
+                # and advance one second
+                self.nc += tod.ONE
+            else:
+                print(u'Timeout called early: ' + ntime.rawtime())
+                #self.log.debug(u'Timeout called early: ' + ntime.rawtime())
+                # no need to advance, desired timeout not yet reached
+        except Exception as e:
+            print(u'Timeout: ' + unicode(e))
+            #self.log.error(u'Timeout: ' + unicode(e))
+
+        # 3: Re-Schedule
+        tt = tod.tod(u'now')+tod.tod(u'0.01')
+        while self.nc < tt:     # ensure interval is positive
+            if tod.MAX - tt < tod.ONE:
+                #self.log.debug(u'Midnight rollover.')
+                break
+            #self.log.debug(u'May have missed an interval, catching up.')
+            self.nc += tod.ONE  # 0.01 allows for processing delay
+        ival = int(1000.0 * float((self.nc - tod.tod(u'now')).timeval))
+        glib.timeout_add(ival, self.timeout)
+
+        # 4: Return False
+        return False    # must return False
+
+    def process_timeout(self):
+        """Perform required timeout activities."""
+        if self.elapstart is not None:
+            estr = u''
+            dstr = u''
+            if self.elapfin is not None:
+                # Race over - show elap and down if poss
+                estr = (self.elapfin - self.elapstart).rawtime(0)
+                if self.timerstat != u'finished':
+                    if self.tod > self.elapfin:
+                        dstr = u'+' + (self.tod - self.elapfin).rawtime(0)
+            else:
+                # race in progress, show run time and distance or lap
+                estr = (self.tod -
+                        (self.elapstart+tod.tod('0.5'))).rawtime(0)
+                if self.distance is not None and self.distance > 0.5:
+                    dstr = u'~{0:1.1f}km'.format(self.distance)
+                if self.lapfin is not None:
+                    # lap down time overwrites dist, but only if valid
+                    laptm = self.tod - self.lapfin
+                    if laptm < self.maxlaptime:
+                        dstr = u'+' + laptm.rawtime(0)
+            self.elap_lbl.set_text(estr)
+            self.gap_lbl.set_text(dstr)
+
     def __init__(self):
         self.io = uscbio()
         self.io.setcb(self.msg_cb)
@@ -606,6 +710,26 @@ class rr_announce(object):
         self.vscroll = ts.get_vadjustment()
         b.connect_signals(self)
 
+        # animation variables
+        self.final = False
+        self.elapstart = None
+        self.elapfin = None
+        self.finstr = None
+        self.timerstat = u'running'     # default assume run state
+        self.distance = None
+        self.laplbl = None
+        self.laptype = None
+        self.lapstart = None
+        self.lapfin = None
+
+        # start timer
+        self.tod = tod.tod(u'now').truncate(0)
+        self.nc = self.tod + tod.tod(u'1.22') # set interval a little off mark
+        self.maxlaptime = tod.tod('2:00') # default maximum lap time
+        #self.log.debug(u'Starting clock intervals at: ' + self.nc.rawtime(3))
+        glib.timeout_add(2000, self.timeout)
+
+
 def main():
     """Run the announce application."""
     metarace.init()
