@@ -19,7 +19,7 @@
 
 This module provides a class 'sportif' which implements the sportif
 ride handler. Written for the 2010 'Ride the Worlds' sportif, and modified
-for the Netti Challenge series, it implements only the required
+for the 2012 Netti Challenge series, it implements only the required
 methods to fit into the roadmeet framework.
 
 """
@@ -56,16 +56,17 @@ COL_SORTKEY = 4
 COL_TEAM = 5
 
 # timing infos
-COL_START = 6		# one-off start time or None for mass start
-COL_RFTIME = 7		# one-off finish time (if relevant) by rfid
+COL_START = 6		# Rider's recorded departure or None for un-start
+COL_RFTIME = 7		# recorded return time
 COL_RFSEEN = 8		# list of tods this rider 'seen' by rfid
 
 # rider commands
-RIDER_COMMANDS_ORD = ['add', 'del', 'que', 'com']
+RIDER_COMMANDS_ORD = ['que', 'add', 'del', 'dnf', 'com']
 RIDER_COMMANDS = {'add':'Add riders',
                   'del':'Delete riders',
                   'que':'Query riders',
-                  'com':'Add comment' }
+                  'com':'Add comment',
+                  'dnf':'Did not Finish' }
 
 # timing keys
 key_armstart = 'F5'
@@ -77,7 +78,7 @@ key_raceover = 'F10'
 key_abort = 'F5'
 
 # config version string
-EVENT_ID = 'sportif-1.4'
+EVENT_ID = 'sportif-1.5'
 
 def key_bib(x):
     """Sort on bib field of aux row."""
@@ -105,6 +106,7 @@ class sportif(object):
                                           u'categories':[],
                                           u'minimums':[],
                                           u'maximums':[],
+                                          u'timerstat':u'idle',
                                           u'finished':False,
                                           u'id':EVENT_ID,
                                           u'startlist':[]}})
@@ -169,7 +171,15 @@ class sportif(object):
         self.set_finish(cr.get(u'event', u'finish'))
         if strops.confopt_bool(cr.get(u'event', u'finished')):
             self.set_finished()
+        else:
+            self.timerstat = u'idle'
+            timereq = cr.get(u'event', u'timerstat')
+            if timereq == 'armstart':
+                self.armstart()
+            elif timereq == 'armfinish':
+                self.armfinish()
 
+        self.recalculate()
         # After load complete - check config and report. This ensures
         # an error message is left on top of status stack. This is not
         # always a hard fail and the user should be left to determine
@@ -208,6 +218,7 @@ class sportif(object):
         if self.finish is not None:
             cw.set(u'event', u'finish', self.finish.rawtime())
         cw.set(u'event', u'finished', self.timerstat == 'finished')
+        cw.set(u'event', u'timerstat', self.timerstat)
         cw.set(u'event', u'startlist', self.get_startlist())    
         catlist = []
         minlist = []
@@ -256,6 +267,10 @@ class sportif(object):
         """Hide event container."""
         self.frame.hide()
 
+    def title_stats_recalc_clicked_cb(self, button, entry=None):
+        """Force recalc of meet stats."""
+        self.recalculate()
+
     def title_close_clicked_cb(self, button, entry=None):
         """Close and save the race."""
         self.meet.close_event()
@@ -462,6 +477,9 @@ class sportif(object):
             for bib in rlist.split():
                 self.query_rider(bib)
             return True
+        elif acode == 'dnf':
+            self.dnfriders(strops.reformat_biblist(rlist))
+            return True
         elif acode == 'com':
             self.add_comment(rlist)
             return True
@@ -469,6 +487,23 @@ class sportif(object):
             self.log.error(u'Ignoring invalid action.')
         return False
 
+    def dnfriders(self, biblist='', code='dnf'):
+        """Remove each rider with supplied code."""
+        recalc = False
+        for bib in biblist.split():
+            r = self.getrider(bib)
+            if r is not None:
+                #r[COL_INRACE] = False
+                r[COL_COMMENT] = code
+                recalc = True
+                self.log.info('Rider ' + str(bib)
+                               + ' did not finish with code: ' + code)
+            else:
+                self.log.warn('Unregistered Rider ' + str(bib) + ' unchanged.')
+        if recalc:
+            self.recalculate()
+        return False
+
     def startlist_gen(self, cat=''):
         """Generator function to export a startlist."""
         mcat = self.ridercat(cat)
@@ -512,7 +547,8 @@ class sportif(object):
 
     def clear_results(self):
         """Clear all data from event model."""
-        self.log.debug(u'Clear results not implemented.')
+        pass	# not relevant for sportif
+        #self.log.debug(u'Clear results not implemented.')
 
     def getrider(self, bib):
         """Return reference to selected rider no."""
@@ -568,16 +604,13 @@ class sportif(object):
         
     def armstart(self):
         """Process an armstart request."""
-        if self.timerstat == 'idle':
+        if self.timerstat in ['idle','running']:
             self.timerstat = 'armstart'
             uiutil.buttonchg(self.meet.stat_but, uiutil.bg_armstart,
                                     'Arm Start')
         elif self.timerstat == 'armstart':
             self.timerstat = 'idle'
             uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Idle') 
-        elif self.timerstat == 'running':
-            # Possible extra state transition here in response to F5
-            pass
 
     def armfinish(self):
         """Process an armfinish request."""
@@ -676,9 +709,89 @@ class sportif(object):
                     self.log.error(u'Duplicate finish rider = ' + repr(bib)
                                       + u' @ ' + e.rawtime(1))
 
+    def new_start_trigger(self, rfid):
+        """Collect a RFID trigger signal and apply it to the model."""
+        if self.newstartdlg is not None and self.newstartent is not None:
+            nt = tod.tod('now')
+            et = tod.str2tod(self.newstartent.get_text())
+            if et is not None:
+                st = rfid - et
+                lt = nt - et
+                self.set_start(st, lt)
+                self.newstartdlg.response(1)
+                self.newstartdlg = None # try to ignore the 'up' impulse
+            else:
+                self.log.warn('Invalid elapsed time: Start not updated.')
+        return False
+
+    def new_start_trig(self, button, entry=None):
+        """Use the 'now' time to update start offset."""
+        self.meet.timer.trig(refid='0')
+
+    def verify_timent(self, entry, data=None):
+        et = tod.str2tod(entry.get_text())
+        if et is not None:
+            entry.set_text(et.rawtime())
+        else:
+            self.log.info('Invalid elapsed time.')
+
+    def elapsed_dlg(self, addriders=''):
+        """Run a 'new start' dialog."""
+        if self.timerstat == 'armstart':
+            self.log.error('Start is armed, unarm to add new start time.')
+            return
+
+        b = gtk.Builder()
+        b.add_from_file(os.path.join(metarace.UI_PATH, 'new_start.ui'))
+        dlg = b.get_object('newstart')
+        dlg.set_transient_for(self.meet.window)
+        self.newstartdlg = dlg
+
+        timent = b.get_object('time_entry')
+        self.newstartent = timent
+        timent.connect('activate', self.verify_timent)
+
+        self.meet.timer.setcb(self.new_start_trigger)
+        b.get_object('now_button').connect('clicked', self.new_start_trig)
+
+        response = dlg.run()
+        self.newstartdlg = None
+        if response == 1:       # id 1 set in glade for "Apply"
+            self.log.info('Start time updated: ' + self.start.rawtime(2))
+        else:
+            self.log.info('Set elapsed time cancelled.')
+        self.meet.timer.setcb(self.rfidtrig)
+        dlg.destroy()
+
     def recalculate(self):
         """Update any internal structures and trigger export if required."""
-        pass
+        tcnt = 0	# Total
+        scnt = 0	# Started
+        uscnt = 0	# Un-Started
+        fcnt = 0	# Finished
+        acnt = 0	# DNF
+        for r in self.riders:
+            tcnt += 1
+            if r[COL_COMMENT]:	# assume dnf/dns - overrides start/finish
+                acnt += 1
+            elif r[COL_RFTIME] is not None:
+                fcnt += 1
+                if r[COL_START] is None:	# un-started
+                    uscnt += 1
+            elif r[COL_START] is not None:
+                scnt += 1
+
+        statmsg = unicode(tcnt) + u' riders'
+        if fcnt > 0:
+            statmsg += u'; ' + unicode(fcnt) + u' finished'
+            if uscnt > 0:
+                statmsg += u'; ' + unicode(uscnt) + u' missed at start'
+        if acnt > 0:
+            statmsg += u'; ' + unicode(acnt) + u' dnf'
+        if scnt > 0:
+            statmsg += u'; ' + unicode(scnt) + u' on course'
+        statmsg += u'.'
+        self.stats_lbl.set_text(statmsg)
 
     def timeout(self):
         """Update elapsed time and recalculate if required."""
@@ -723,14 +836,15 @@ class sportif(object):
 
     def set_elapsed(self):
         """Update the elapsed time field."""
-        if self.start is not None and self.finish is not None:
-            self.time_lbl.set_text((self.finish - self.start).timestr(0))
-        elif self.start is not None:    # Note: uses 'local start' for RT
-            self.time_lbl.set_text((tod.tod('now') - self.lstart).timestr(0))
-        elif self.timerstat == 'armstart':
-            self.time_lbl.set_text(tod.tod(0).timestr(0))
-        else:
-            self.time_lbl.set_text('')
+        pass
+        #if self.start is not None and self.finish is not None:
+            #self.time_lbl.set_text((self.finish - self.start).timestr(0))
+        #elif self.start is not None:    # Note: uses 'local start' for RT
+            #self.time_lbl.set_text((tod.tod('now') - self.lstart).timestr(0))
+        #elif self.timerstat == 'armstart':
+            #self.time_lbl.set_text(tod.tod(0).timestr(0))
+        #else:
+            #self.time_lbl.set_text('')
 
     def set_running(self):
         """Update event status to running."""
@@ -796,7 +910,7 @@ class sportif(object):
         st = model.get_value(iter, COL_START)
         otxt = ''
         if st is not None:
-            otxt = st.rawtime(2)
+            otxt = st.rawtime(0)
         cr.set_property('text', otxt)
 
     # show finish offset
@@ -804,7 +918,7 @@ class sportif(object):
         ft = model.get_value(iter, COL_RFTIME)
         otxt = ''
         if ft is not None:
-            otxt = ft.rawtime(2)
+            otxt = ft.rawtime(0)
         cr.set_property('text', otxt)
 
     # show elapsed time
@@ -812,7 +926,7 @@ class sportif(object):
         et = self.get_rider_elapsed(iter)
         otxt = ''
         if et is not None:
-            otxt = et.rawtime(2)
+            otxt = et.rawtime(0)
         cr.set_property('text', otxt)
 
     # edit start
@@ -847,6 +961,10 @@ class sportif(object):
         self.mintime = tod.tod('5:00')		# abs min
         self.maxtime = tod.tod('20h00:00')	# abs max
 
+        # new start dialog
+        self.newstartent = None
+        self.newstartdlg = None                                                 
+
         self.riders = gtk.ListStore(gobject.TYPE_STRING, # BIB = 0
                                     gobject.TYPE_STRING, # NAMESTR = 1
                                     gobject.TYPE_STRING, # CAT = 2
@@ -867,7 +985,7 @@ class sportif(object):
         # meta info pane
         self.title_namestr = b.get_object('title_namestr')
         self.set_titlestr()
-        self.time_lbl = b.get_object('time_lbl')
+        self.stats_lbl = b.get_object('stats_lbl')
 
         # results pane
         t = gtk.TreeView(self.riders)
